from robolink import *
import socket
import json
from time import sleep

class MCenv(Robolink):
    
    def __init__(self):
        #Initilize the same way Robolink does
        #Plus intializing 2 class parameters
        #robotDic - A dictionary containing the robot names given by the user as keys
        #and robot items as values
        #sockMC - The socket channel used for communication with MC
        super(MCenv,self).__init__()
        self.robotDic = {}
        self.sockMC = None
    
    def setRobot(self, userRobotName, roboDKrobotName = None):
        #Iniializes an Item of type robot
        #userRobotName - A name the user choses and it should
        #be the same as in MC.
        #roboDKrobotName - The name of the robot in roboDK environment
        #If no name is given, the first name in the list of availale robots is chosen
    
        self.itemList = self.ItemList(ITEM_TYPE_ROBOT)
        if roboDKrobotName is not None:
            if roboDKrobotName in self.itemList:
                robotNum = self.itemList.index(roboDKrobotName) + 1
            else:
                robotNum = 1
                print('Robot Name not found, openning robot number: ', robotNum)

        try:
            roboDKrobotName = self.itemList[robotNum-1].Name() # good for versions 3.3 and above
        except AttributeError:
            roboDKrobotName = self.itemList[robotNum-1] # good for versions below 3.3

        tmpRobot = self.Item(roboDKrobotName, ITEM_TYPE_ROBOT)
        print('Name of selected robot in roboDK: ' + roboDKrobotName)
        self.robotDic[userRobotName] = tmpRobot
        self.writeStr("ready")
        jsonDic = self._json2Dic([userRobotName], self.readJsonStr())
        tmpRobot.setJoints(jsonDic[userRobotName])
        #return robotName

    def _json2Dic(self, robotList, jsonStr):
        #Format: '{"a1": double, "a2": double, "a3": double, "a4": double, "a5": double, "a6": double}'
        #Converts a JSON string into dictionary
        try:
            jsonDic = json.loads(jsonStr)
        except ValueError:
            print("Probably blank line input - setting last joints values")
            jsonDic = {}
            for userRobotName in robotList:
                jsonDic[userRobotName] = self.robotDic[userRobotName].Joints()
        return jsonDic
        #Same code as above in one line, without 'try':
        #return list(json.loads(jsonStr).values())

    def moveRobots(self, robotList, jsonStr, method='J', blocking=True):
        #Moves the robots that exists both in roboDK list and in MC list
        jsonDic = self._json2Dic(robotList, jsonStr)
        mcRobotList = list(jsonDic.keys())
        for userRobotName in robotList:
            if userRobotName in mcRobotList:
                if method == 'J':
                    self.robotDic[userRobotName].MoveJ(jsonDic[userRobotName],blocking)
                else:
                    self.robotDic[userRobotName].MoveL(userRobotName[userRobotName],blocking)

    def connectClient(self, host,port,timeout=0.1):
        self.sockMC = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        self.sockMC.settimeout(timeout)
        #try:
        #print("client tries to connect")
        self.sockMC.connect((host,port))
        print("client is connected")
        sleep(0.5)
        #except:
            #print("failed connecting")
        #print("Sending 'start'")
        self.sockMC.send("start".encode()) # let know client is ready
        #print("'start' was sent")
        sleep(0.5)
        
    def readJsonStr(self, numOfBytes=256):
        try:
            return self.sockMC.recv(numOfBytes).decode()
        except socket.timeout:
            return ''
    
    def writeStr(self, inputStr):
        self.sockMC.send(inputStr.encode())
        
